<?php
/**** PHP开发接口专业版v0.2.1说明 ****
 *本代码仅适用于金笛短信WEB中间件http接口，基于web中间件《开发接口说明文档》编写
 *
 **** 使用方法 ****
 *1.按照注释配置[用户参数设置区]的各项参数
 *2.用测试工具测试当前配置下是否正常将消息插入到待发送队列
 *3.将本代码适配后，加入到您的项目中
 *
 **** 变量说明 ****
 *    变量            含义        数据类型        说明                                    举例
 *    $recipient        收信人        数组或字符串    对方手机号,多个联系人用英文逗号分隔        "138001380138,138001380139,18901890189"
 *    $message        短信内容    字符串            支持中文短信内容，程序自动编码            "Good good study,day day up"
 *    $debug            调试模式    布尔或整型        默认不开启,调试模式设置为1后开启        调试模式开启后可以在函数返回值中输出主要操作进度
 *****
 *常见问题
 *1.SSL无法访问：
 *  PHP中一般无法直接用file_get_contents获取SSL加密的内容，所以选择cURL来传递数据，使用时注意将SSL验证关闭
 *  curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
 *  curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
 *2.访问显示乱码
 *  需要使用https访问中间件的接口
 *3.接口调用出现错误
 *  返回的错误参数说明  -9000：参数非法，-9001：缺少参数， -9002：发送错误，-9003：密码错误， -9999：发送终止 ;
 *4.无法将新数据加入到待发送列表中
 *  (1)确保中间件的数据库可用并已连上(详见数据源设置页面)
 *  (2)http接口调用时添加了不适当的参数
 *5.更改http API的端口号
 *  端口可以在配置文件中更改，在安装目录conf/JDSMSServer.conf中，httpServer.port参数的值即为API的端口
 *
 *6.短信发不出去
 *  (1)检查待发送列表是否有数据
 *  (2)直接访问API的网址，查看是否有返回
 *  (3)直接在中间件发送一条短信，并在[设备配置]或[短信设备]中确认设备处于[启用]状态
 *  (4)设备显示[未授权]时,中间件可用于测试，但会随机附带"金笛短信为您服务"，而且有88条的数量限制，请留意是否已用完测试限额
 *
 *7.发出去的短信乱码\待发送列表中短信内容显示乱码
 *  一般是编码问题，请将发出去的文字编码设置为UTF-8
 *
 *8.发出去的短信多了"金笛短信为您服务"的字样
 *  说明您的中间件尚未注册，进入实际生产环境阶段之前请将设备
 *****
 *建议：在开发期间，只要待发送列表能显示插入的消息，即可确认接口程序成功建立连接，故建议不要在调试期间开启设备
 ****
 *另附:高版本中间件可以一次发送多条短信，收件人用";"分隔后以字符串形式输入即可，如"138001380138;138001380139;18901890189",中间件会自动处理
 ***
 * @param $recipient
 * @param $message
 * @param int $debug
 * @return int|string debuginfo
 */

function send_sms($recipient,$message,$debug=0){		
		
	/**** 用户参数设置区开始 ****/

	/**** 中间件参数 ****/
	$midware_ip			= "127.0.0.1";			//中间件所在服务器的IP
	$midware_port		= "8060";				//中间件发信API端口(默认8060)
	$midware_utf8encode	= false;				//是否开启UTF8编码, 如出现乱码请开启
	$midware_SSL		= true;					//SSL协议 			true:已开启	false:未开启 (一般为未开启，3.6.8版本以后默认开启)
	$test_content		= "您好！\n欢迎使用金笛短信WEB中间件！";			//发送测试短信时发送的内容
	
	/**** 发信参数 ****/
	$send_priority	= 0;						//发信优先级 低：负数 普通：0 高：正数

	/**** 用户参数设置区结束 ****/		
	

	$debug_log = "Debug: Preparing to send SMS\n";
	$error_log = '';
	$sms_count = 0;
	$linux_env = (strtoupper(substr(PHP_OS,0,3))==='WIN')?0:1;
	$err_msg   = array(
		-9000  => '[SMS] sender illegal param'	,
		-9001  => '[SMS] sender lack of param'	,
		-9002  => '[SMS] send error'			,
		-9003  => '[SMS] wrong password'		,
		-9999  => '[SMS] sending aborted'      	,
	);/**参数说明  -9000：参数非法，-9001：缺少参数， -9002：发送错误，-9003：密码错误， -9999：发送终止 ; **/
	
	if(is_array($recipient)){		
		$recipient_array = $recipient;
	}else{
		$recipient_array = explode(',', $recipient);	
	}
	if(!$message){
		$message = $test_content;
	}
	if(!$midware_utf8encode){
		$text = urlencode($message);
	}else{
		$text = urlencode(utf8_encode($message));
	}				
		
	$debug_log.="Debug: New SMS to [{$recipient_array[0]}]:raw[{$message}]:encoded[{$text}] \n";
	
	foreach ($recipient_array as $to) {
		
		if(!$to){
			$debug_log.="Debug: Skipping null phone number \n---\n";
			continue;					//遇到空数据跳出循环
		}
		$to=trim($to);			
		$sms_count=$sms_count+1;
		$recipient=$to;
		$protocol=$midware_SSL?"https":"http";
		$midware_link ="{$protocol}://{$midware_ip}:{$midware_port}";
		$midware_password=$midware_password?$midware_password:1;
		$midware_url = "{$midware_link}/send?password={$midware_password}&text={$text}&recipient={$recipient}&encoding=U&priority={$send_priority}"; 
		$debug_log.="Debug: SMS[{$sms_count}] - Parsed url is {$midware_url} \n";
		if($midware_xml = geturl($midware_url,$midware_SSL)){

			$debug_log.="Debug: SMS[{$sms_count}] - Successfully sent data to SMS midware\n";
			$midware_dom 	= DOMDocument::loadXML($midware_xml);		 
			$midware_error	= $midware_dom->getElementsByTagName("error")->item(0)->textContent;

			if($midware_error=='0'){
				
				$midware_status	= $midware_dom->getElementsByTagName("message_status")->item(0)->textContent;
				$midware_msg_id	= $midware_dom->getElementsByTagName("msg_id")->item(0)->textContent;
				if($midware_status=='UNSENT' or $midware_status=='SENT'){
					
					$debug_log.="Debug: SMS[{$sms_count}] - SMS to {$recipient} is successfully added to midware queue list, midware replyed: [$midware_status]\n---\n";

				}else{
					
					$midware_failure_cause	= @$midware_dom->getElementsByTagName("failure_cause")->item(0)->textContent;
					if(!$midware_failure_cause){
						$midware_failure_cause='UNKOWN';
					}
					$error_log=$debug_log."WARNING: Sending SMS To {$recipient} Failed.  Caused by {$midware_failure_cause}";
					return $error_log;
				}
			}else{

				$midware_failure_cause	= $midware_dom->getElementsByTagName("error_description")->item(0)->textContent;
				$error_log=$debug_log."WARNING: Sending SMS To {$recipient} Failed.  Error was [{$midware_error}]:{$err_msg[$midware_error]} , caused by ".$midware_failure_cause?$midware_failure_cause:"[Notdefined]";
				return $error_log;
			}
		}else{
			$error_log=$debug_log."ERROR: Sending SMS To [". $recipient . "] Failed.  Error was Cannot get right response form midware URL- <a href={$midware_link}>{$midware_link}</a>";
			return $error_log;
		}
	}
	$debug_log.="Debug: DONE -  Send list Completed";
	return $debug?$debug_log:0;
}
	
function geturl($url,$SSL=false){
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	if($SSL){
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
	}
	$output = curl_exec($ch);
	curl_close($ch);
	return $output;
}	